var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Api } from "../../Api";
import { LoggerService } from "../Logging/LoggerService";
import { DataFrame, Method, Status } from "../../DataFrame";
import { ExalusConnectionService } from "../ExalusConnectionService";
export class DeviceUpdateProvider {
    constructor() {
        this._services = new Map();
        this._runtimeInfo = new SoftwareRuntimeInfo();
        this._isRuntimeInfoInitialized = false;
    }
    GetServiceName() {
        return DeviceUpdateProvider.ServiceName;
    }
    GetSoftwareRuntimeInfoAsync(reloadCachedData = false) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this._isRuntimeInfoInitialized || reloadCachedData) {
                yield this.GetRuntimeControllerInfoAsync();
                return this._runtimeInfo;
            }
            return this._runtimeInfo;
        });
    }
    RegisterUpdateProvider(service) {
        Api.Get(LoggerService.ServiceName).Debug(DeviceUpdateProvider.ServiceName, `Registering new update provider: ${service.GetUpdateProviderName()}`);
        this._services.set(service.GetUpdateProviderName(), service);
    }
    GetUpdateProviderAsync(serviceName, forceReloadExtensions = false) {
        return __awaiter(this, void 0, void 0, function* () {
            Api.Get(LoggerService.ServiceName).Debug(DeviceUpdateProvider.ServiceName, `Getting update provider [${serviceName}] by GetUpdateProviderByName<T>()`);
            const service = this._services.get(serviceName);
            if (service == null)
                throw new DeviceUpdateProviderNotFound("Cannot get update provider! Requested update provider has not been registered.");
            return service;
        });
    }
    GetUpdateProvidersAsync(param, apiVersion) {
        return __awaiter(this, void 0, void 0, function* () {
            const regGuid = new RegExp("^[{]?[0-9a-fA-F]{8}-([0-9a-fA-F]{4}-){3}[0-9a-fA-F]{12}[}]?$");
            if (regGuid.test(param)) {
                Api.Get(LoggerService.ServiceName).Debug(DeviceUpdateProvider.ServiceName, `Getting update provider with protocol guid: [${param}] by GetUpdateProvidersByProtocolGuid<T>()`);
                let services = [];
                if (apiVersion != null)
                    services = [...this._services.values()].filter(provider => provider.ProtocolGuid == param && provider.AvailableExtensionVersion == apiVersion);
                else
                    services = [...this._services.values()].filter(provider => provider.ProtocolGuid == param);
                if (services.length == 0)
                    throw new DeviceUpdateProviderNotFound("Cannot get update provider! Requested update provider has not been registered.");
                return services;
            }
            else
                throw new DeviceUpdateProviderNotFound("Cannot get update provider! Bad parameters.");
        });
    }
    GetRuntimeControllerInfoAsync() {
        return __awaiter(this, void 0, void 0, function* () {
            Api.Get(LoggerService.ServiceName).Debug(DeviceUpdateProvider.ServiceName, `Getting runetime version info from controller.`);
            const result = yield Api.Get(ExalusConnectionService.ServiceName).SendAndWaitForResponseAsync(new GetControllerRuntimeInfoRequest(), 16000, false);
            if (result == null || result.Status == null)
                throw new CannotGetRuntimeInfo(`Cannot get runtime info, controller response result is empty."`);
            if (result.Status != Status.OK)
                throw new CannotGetRuntimeInfo(`Cannot get runtime info, controller responded with status ${result.Status}.`);
            if (result.Data == null)
                throw new CannotGetRuntimeInfo(`Cannot get runtime info, controller responded with status OK but response does not contains data.`);
            this._runtimeInfo = result.Data;
            this._isRuntimeInfoInitialized = true;
        });
    }
}
DeviceUpdateProvider.ServiceName = "DeviceServiceProvider";
//RuntimeInfo
class GetControllerRuntimeInfoRequest extends DataFrame {
    constructor() {
        super();
        this.Resource = "/controller/software/info";
        this.Method = Method.Get;
    }
}
export class SoftwareRuntimeInfo {
    constructor() {
        this.BaseLinuxVersion = "";
        this.BaseMonoVersion = "";
        this.RuntimeVersion = "";
        this.UpdateChannel = "";
        this.SoftwareVersion = "";
    }
}
//Errors
export class DeviceUpdateProviderNotFound extends Error {
    constructor(message) {
        super(message);
        this.message = message;
        this.name = "DeviceUpdateProviderNotFound";
    }
}
export class CannotGetRuntimeInfo extends Error {
    constructor(message) {
        super(message);
        this.message = message;
        this.name = "CannotGetRuntimeInfo";
    }
}
//# sourceMappingURL=DeviceUpdateProvider.js.map